#include "../../shared/map/map.h"

#include "train.h"

#include "../common.h"

train::train()
{
	//TRACE("Map W: %i H: %i", (*level)[0]->get_width(), (*level)[0]->get_height());

	x = x_last = 0;
	y = y_last = 0;

	// Get the starting position:
	for(int mx = 0; mx < (*map_c)[0]->get_width(); mx++)
	{
		for(int my = 0; my < (*map_c)[0]->get_height(); my++)
		{
			//TRACE("X: %i Y: %i Value: %i", mx, my, level->get(1, mx, my));
			if(map_c->get(1, mx, my) == 107)
			{
				x = x_last = mx;
				y = y_last = my;
			}
		}
	}

	x_fine = y_fine = 0;
	dir = DIR_LEFT;

}


train::~train()
{

}

bool train::is_in_goal()
{
  if(x_fine == 0 && y_fine == 0 && map_c->get(1, x, y) == 106)
   return true;
  else
   return false;
}

bool train::is_crashed()
{
  if(x_fine == 0 && y_fine == 0 && dir == DIR_NONE)
   return true;
  else
   return false;
}



bool train::update()
{
 
 //TRACE("Train - update: Trying to find a place to move to are at %i, %i now\n", x, y);

 // Move to new square:
 if(x_fine == 0 && y_fine == 0)
 {
		// Try going forward first:
		if(can_move_to(dir_forw_rel(dir)))
		{
			move_to(dir_forw_rel(dir));
		}
		// Then to the sides:
		else if(can_move_to(dir_right_rel(dir)))
		{
			move_to(dir_right_rel(dir));
		}
		else if(can_move_to(dir_left_rel(dir)))
		{
			move_to(dir_left_rel(dir));
		}
		// Then down:
		// else if(can_move_to(level, tracklist, dir_back_rel(dir)))
		// {
		// 	move_to(dir_back_rel(dir));
		// }
		// Stuck?
		else
		{
			dir = DIR_NONE;
		}
	

 }

  // Small steps towards the goal;
	switch(dir)
	{
		case DIR_UP:    y_fine--; break;
		case DIR_DOWN:  y_fine++; break;
		case DIR_LEFT:  x_fine--; break;
		case DIR_RIGHT: x_fine++; break;
	}

	return true;
}



bool train::render()
{
	BITMAP *train;

	switch(dir)
	{
		default: case DIR_NONE:  train = tileset_c->tile[100]; break;
		case DIR_UP:    train = tileset_c->tile[97]; break;
		case DIR_DOWN:  train = tileset_c->tile[96]; break;
		case DIR_LEFT:  train = tileset_c->tile[99]; break;
		case DIR_RIGHT: train = tileset_c->tile[98]; break;
	}

  //textprintf_ex(buffer, font, 10, 10, makecol(255, 255, 255), 0, "Train X: %i Y: %i (%i, %i), Dir: %", x, y, x_last, y_last, dir);


  //textprintf_ex(buffer, font, (x) * TILE_W, (y) * TILE_H, makecol(255, 255, 255), 0, "X");

  //if(x > 0 && y > 0)
	masked_blit(train, buffer, 0, 0, x * TILE_W + x_fine, y * TILE_H + y_fine, SCREEN_W, SCREEN_H);

	return true;
}

bool train::move_to(int direction)
{

	if(direction == DIR_UP)
	{
  	y_last = y--;	x_last = x; dir = DIR_UP; y_fine = TILE_H;
  }
	else if(direction == DIR_DOWN)
	{
		y_last = y++;	x_last = x; dir = DIR_DOWN; y_fine = -TILE_H;
	}
	else if(direction == DIR_LEFT)
	{
		x_last = x--;	y_last = y; dir = DIR_LEFT; x_fine = TILE_W;
	}
	else if(direction == DIR_RIGHT)
	{
		x_last = x++;	y_last = y; dir = DIR_RIGHT; x_fine = -TILE_W;
	}
  else
  {
   return false;
  }
 return true;
}



bool train::can_move_to(int direction)
{
 int track_cur = map_c->get(1, x, y);
 int track_nex = -1;

 if(direction == DIR_UP)
 {
 	int track_nex = map_c->get(1, x, y - 1);
	 TRACE("Train - can_move_to: Trying to find out if we can move in dir %i at %i now.. %i after\n", direction, track_cur, track_nex);
	if((*tracklist)[track_cur].up() && (*tracklist)[track_nex].down())return true;
 }
 else if(direction == DIR_RIGHT)
 {
 	int track_nex = map_c->get(1, x + 1, y);
	 TRACE("Train - can_move_to: Trying to find out if we can move in dir %i at %i now.. %i after\n", direction, track_cur, track_nex);
	if((*tracklist)[track_cur].right() && (*tracklist)[track_nex].left())return true;
 }
 else if(direction == DIR_DOWN)
 {
 	int track_nex = map_c->get(1, x, y + 1);
	 TRACE("Train - can_move_to: Trying to find out if we can move in dir %i at %i now.. %i after\n", direction, track_cur, track_nex);
	if((*tracklist)[track_cur].down() && (*tracklist)[track_nex].up())return true;
 }
 else if(direction == DIR_LEFT)
 {
 	int track_nex = map_c->get(1, x - 1, y);
	 TRACE("Train - can_move_to: Trying to find out if we can move in dir %i at %i now.. %i after\n", direction, track_cur, track_nex);
	if((*tracklist)[track_cur].left() && (*tracklist)[track_nex].right())return true;
 }

 if(track_nex != -1)
 { 

 }
 TRACE("Train - can_move_to: Dang (%i), failed\n", direction);


 return false;
}


int train::dir_left_rel(int direction)
{
 direction--;
 if(direction == 0)direction = 4;
 return direction;
}
int train::dir_right_rel(int direction)
{
 direction++;
 if(direction == 5)direction = 1;
 return direction;
}
int train::dir_back_rel(int direction)
{
 switch(direction)
 {
	case DIR_UP: return DIR_DOWN;
  case DIR_RIGHT: return DIR_LEFT;
  case DIR_DOWN: return DIR_UP;
  case DIR_LEFT: return DIR_RIGHT;
	default: return DIR_NONE;
 }
}
int train::dir_forw_rel(int direction)
{
 return direction;
}



