#include "allegro.h"

#include "map.h"



map::map()
{
 TRACE("Map - Constructor reached\n");
  
}

map::~map()
{
 TRACE("Map - Deconstructor reached\n");
 destroy();
}


bool map::layer_create(int w, int h, int t, int a)
{
 TRACE("Map - layer_create: Trying to create layer (%i x %i)\n", w, h);
 
 layer *tmp = new layer(0, 0, w, h, t, a);

 layers.push_back(tmp);

 TRACE("Map - layer_create: Done creating layer\n");

 return true;
}

bool map::layer_delete(unsigned int i)
{
 TRACE("Map - layer_delete: Deleteing layer %i\n", i);
 if(layer_exists(i))
 {
  delete layers[i];
  layers.erase(layers.begin() + i);

  return true;
 }
 return false;
}

bool map::layer_exists(unsigned int i)
{
 if(i >= 0 && i < layers.size())
 {
  return true;
 }
 return false;
}


bool map::save(const char *filename)
{
 TRACE("Map - save: Reached, trying to open map %s for writing... ", filename); 
 
 PACKFILE *file = pack_fopen(filename, "w");
 
 if(file)
 {
  TRACE("Done!\n");
  
  pack_iputw(layers.size(), file);
  
  TRACE("Map - save: Done writing map header\n");

  for(unsigned int l = 0; l < layers.size(); l++)
  {
   pack_iputw(layers[l]->pos_x, file);   
   pack_iputw(layers[l]->pos_y, file);  

   pack_iputw(layers[l]->width, file);   
   pack_iputw(layers[l]->height, file);   

   pack_iputw(layers[l]->type, file);   
   pack_iputw(layers[l]->alpha, file);

  TRACE("Map - save: Done writing layer header (layer %i)\n", l);    

   for(int x = 0; x < layers[l]->width; x++)
    for(int y = 0; y < layers[l]->height; y++)
     pack_iputw(layers[l]->data[x][y], file);

  TRACE("Map - save: Done writing layer data   (layer %i)\n", l); 
  }  

  TRACE("Map - save: Done saving map\n");
  
  pack_fclose(file);
	return true;
 }
 else
 {
  TRACE("Failed!\n"); 
	return false;
 }
 
 
}

bool map::load(const char *filename)
{
 TRACE("Map - load: Reached, trying to open map %s for reading... ", filename); 
 PACKFILE *file = pack_fopen(filename, "r");
 
 if(file)
 { 
  TRACE("Done!\n");

  destroy();
  
  int layer_count = pack_igetw(file);

  TRACE("Map - load: Done reading header\n");

  for(int l = 0; l < layer_count; l++)
  {
   /*int x =*/ pack_igetw(file);
   /*int y =*/ pack_igetw(file);

   int w = pack_igetw(file);
   int h = pack_igetw(file);

   int t = pack_igetw(file);
   int a = pack_igetw(file);

   TRACE("Map - load: Done reading layer header (layer %i)\n", layers.size() - 1);   
   layer_create(w, h, t, a);

   for(int x = 0; x < layers[layers.size() - 1]->width; x++)
    for(int y = 0; y < layers[layers.size() - 1]->height; y++)
     layers[layers.size() - 1]->data[x][y] = pack_igetw(file);

   TRACE("Map - load: Done reading layer data   (layer %i)\n", layers.size() - 1);   
  }

  TRACE("Map - load: Done reading map\n");
 
  pack_fclose(file);
 
  return true;
 }
 else
 {
  TRACE("Failed!\n");
  return false;
 }
}

bool map::destroy()
{
 TRACE("Map - destroy: Reached\n");
 
 while(layers.size())
 {
  TRACE("Map - destroy: Deleting layer\n");
  layer_delete(0);
 }
 
 TRACE("Map - destroy: Completed\n");
 return true; 
}


bool map::render(BITMAP *target, tileset *t, int camera_x, int camera_y)
{  
 int xo = camera_x / TILE_W;
 int yo = camera_y / TILE_H;	
	
 int xo_small = camera_x % TILE_W;
 int yo_small = camera_y % TILE_H;	
	
 //TRACE("Map - render: Trying to render map (%i, %i)\n", xo, yo);

 for(unsigned int l = 0; l < layers.size(); l++)
 {
  //TRACE("Map - render: Trying to render layer %i\n", l);
  if(layers[l]->alpha != 0 && layers[l]->alpha == 255)
  {
   for(int x = 0; x < SCREEN_W / TILE_W + 1; x++)
    for(int y = 0; y < SCREEN_H / TILE_H + 1; y++)
    {
     //TRACE("Map - render: Trying to render tile (%i, %i) with tile %i \n", x + xo, y + yo, layers[l]->data[x + xo][y + yo]);	    
     draw_sprite(target, t->tile[ layers[l]->data[x + xo][y + yo] ], x * TILE_W - xo_small, y * TILE_H - yo_small);
    } 
  }
  else if(layers[l]->alpha > 0 && layers[l]->alpha < 255)
  {
   set_trans_blender(0, 0, 0, layers[l]->alpha);
   for(int x = 0; x < SCREEN_W / TILE_W; x++)
    for(int y = 0; y < SCREEN_H / TILE_H; y++)
    {
     //TRACE("Map - render: Trying to render (transparent) tile (%i, %i) with tile %i \n", x + xo, y + yo, layers[l]->data[x + xo][y + yo]);	    	    
     draw_trans_sprite(target, t->tile[ layers[l]->data[x + xo][y + yo] ], x * TILE_W, y * TILE_H);
    } 
  }
  else
  {
   //textprintf_ex(target, font, 10, 100 + (10 * l), makecol(255, 255, 255), -1, "No need to draw layer %i", l);
  }

  //textprintf_ex(target, font, 10, 60 + (10 * l), makecol(255, 255, 255), -1, "Layer %i - w: %i, h: %i, type: %i, alpha: %i", l,  layers[l]->width,  layers[l]->height,  layers[l]->type, layers[l]->alpha);
 }
 //TRACE("Map - render: Done!\n");

 return true;
}

bool map::set(unsigned int l, int x, int y, int n)
{
 if(l >= 0 && l < layers.size())
 {
  layers[l]->data[x][y] = n;  
 }
 return -1;
}

int map::get(unsigned int l, int x, int y)
{
 if(l >= 0 && l < layers.size())
 {
  return layers[l]->data[x][y];
 }
 return -1;
}

unsigned int map::layer_count()
{
 return layers.size();

}
